import { NextPageContext, PreviewData, GetServerSidePropsContext, GetServerSideProps, GetStaticPropsContext, GetStaticProps } from 'next';
import { ParsedUrlQuery } from 'querystring';

declare type Gip<T> = (ctx: NextPageContext) => T | Promise<T>;
declare type ComposeInitialPropsParams<T extends unknown[], CompositeT, U = CompositeT> = {
    readonly use: T;
    readonly resolver?: (props: CompositeT, context: NextPageContext) => U | Promise<U>;
};
declare function composeInitialProps<T0, CompositeT = T0, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, CompositeT = T0 & T1, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, CompositeT = T0 & T1 & T2, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, CompositeT = T0 & T1 & T2 & T3, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, CompositeT = T0 & T1 & T2 & T3 & T4, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, CompositeT = T0 & T1 & T2 & T3 & T4 & T5, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, T7, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>, Gip<T7>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, T7, T8, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>, Gip<T7>, Gip<T8>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>, Gip<T7>, Gip<T8>, Gip<T9>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>, Gip<T7>, Gip<T8>, Gip<T9>, Gip<T10>], CompositeT, U>): Gip<U>;
declare function composeInitialProps<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, CompositeT = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10 & T11, U = CompositeT>(params: ComposeInitialPropsParams<[Gip<T0>, Gip<T1>, Gip<T2>, Gip<T3>, Gip<T4>, Gip<T5>, Gip<T6>, Gip<T7>, Gip<T8>, Gip<T9>, Gip<T10>, Gip<T11>], CompositeT, U>): Gip<U>;

declare type Gssp<P extends {
    [key: string]: any;
} = {
    [key: string]: any;
}, Q extends ParsedUrlQuery = ParsedUrlQuery, D extends PreviewData = PreviewData> = GetServerSideProps<P, Q, D>;
declare type ComposeServerSidePropsParams<T extends unknown[], Q extends ParsedUrlQuery, D extends PreviewData, P, ResolvedP = P> = {
    readonly use: T;
    readonly resolver?: (props: P, context: GetServerSidePropsContext<Q, D>) => ResolvedP | Promise<ResolvedP>;
};
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, P = T0, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, P = T0 & T1, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, P = T0 & T1 & T2, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, P = T0 & T1 & T2 & T3, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, P = T0 & T1 & T2 & T3 & T4, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, P = T0 & T1 & T2 & T3 & T4 & T5, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, P = T0 & T1 & T2 & T3 & T4 & T5 & T6, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>, Gssp<T7, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>, Gssp<T7, Q, D>, Gssp<T8, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>, Gssp<T7, Q, D>, Gssp<T8, Q, D>, Gssp<T9, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>, Gssp<T7, Q, D>, Gssp<T8, Q, D>, Gssp<T9, Q, D>, Gssp<T10, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;
declare function composeServerSideProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10 & T11, ResolvedP = P>(params: ComposeServerSidePropsParams<[Gssp<T0, Q, D>, Gssp<T1, Q, D>, Gssp<T2, Q, D>, Gssp<T3, Q, D>, Gssp<T4, Q, D>, Gssp<T5, Q, D>, Gssp<T6, Q, D>, Gssp<T7, Q, D>, Gssp<T8, Q, D>, Gssp<T9, Q, D>, Gssp<T10, Q, D>, Gssp<T11, Q, D>], Q, D, P, ResolvedP>): Gssp<P, Q, D>;

declare type Gsp<P extends {
    [key: string]: any;
} = {
    [key: string]: any;
}, Q extends ParsedUrlQuery = ParsedUrlQuery, D extends PreviewData = PreviewData> = GetStaticProps<P, Q, D>;
declare type RevalidateCompositionType = 'min' | 'max';
declare type ComposeStaticPropsParams<T extends unknown[], Q extends ParsedUrlQuery, D extends PreviewData, P, ResolvedP = P> = {
    readonly use: T;
    readonly resolver?: (props: P, context: GetStaticPropsContext<Q, D>) => ResolvedP | Promise<ResolvedP>;
    readonly revalidate?: number | RevalidateCompositionType;
};
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, P = T0, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, P = T0 & T1, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, P = T0 & T1 & T2, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, P = T0 & T1 & T2 & T3, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, P = T0 & T1 & T2 & T3 & T4, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, P = T0 & T1 & T2 & T3 & T4 & T5, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, P = T0 & T1 & T2 & T3 & T4 & T5 & T6, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>, Gsp<T7, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>, Gsp<T7, Q, D>, Gsp<T8, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>, Gsp<T7, Q, D>, Gsp<T8, Q, D>, Gsp<T9, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>, Gsp<T7, Q, D>, Gsp<T8, Q, D>, Gsp<T9, Q, D>, Gsp<T10, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;
declare function composeStaticProps<Q extends ParsedUrlQuery, D extends PreviewData, T0, T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, P = T0 & T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8 & T9 & T10 & T11, ResolvedP = P>(params: ComposeStaticPropsParams<[Gsp<T0, Q, D>, Gsp<T1, Q, D>, Gsp<T2, Q, D>, Gsp<T3, Q, D>, Gsp<T4, Q, D>, Gsp<T5, Q, D>, Gsp<T6, Q, D>, Gsp<T7, Q, D>, Gsp<T8, Q, D>, Gsp<T9, Q, D>, Gsp<T10, Q, D>, Gsp<T11, Q, D>], Q, D, P, ResolvedP>): Gsp<P, Q, D>;

export { ComposeInitialPropsParams, ComposeServerSidePropsParams, ComposeStaticPropsParams, RevalidateCompositionType, composeInitialProps, composeServerSideProps, composeStaticProps };
